# Generated by Django 3.2.3 on 2021-05-21 09:31

import django.contrib.postgres.fields.ranges
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("pg", "0003_jsonmodel"),
    ]

    operations = [
        migrations.CreateModel(
            name="DateRangeModel",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("date_range", django.contrib.postgres.fields.ranges.DateRangeField()),
            ],
        ),
    ]
